#include <stdlib.h>
#include <stdio.h>
#include "bin_read.h"

/***********************************************************************
 * void float_getbin (FILE* unit, float *var)
 * void float_array_getbin (FILE* unit, float *var, int nr)
 *
 * read binary float (float array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void float_getbin (FILE* unit, float *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    int n;
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(float)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(float), 1, unit);
#endif
}


void float_array_getbin (FILE* unit, float *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    int n;
    unsigned k;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(float)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(float) + n , sizeof(char), 1, unit);
    }
#else
	size = fread (vo, sizeof(float), nr, unit);
#endif	
}
    
/***********************************************************************
 * void double_getbin (FILE* unit, double *var)
 * void double_array_getbin (FILE* unit, double *var, int nr)
 *
 * read binary double (double array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void double_getbin (FILE* unit, double *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

#if defined(__LITTLE_ENDIAN__)
    int n;
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(double)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(double), 1, unit);
#endif
}


void double_array_getbin (FILE* unit, double *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    unsigned k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(double)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(double) + n , sizeof(char), 1, unit);
    }
#else
	size = fread (vo, sizeof(double), nr, unit);
#endif	
}
    

/***********************************************************************
 * void int_getbin (FILE* unit, int *var)
 * void int_array_getbin (FILE* unit, int *var, int nr)
 *
 * read binary int (int array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void int_getbin (FILE* unit, int *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

#if defined(__LITTLE_ENDIAN__)
    int n;
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(int)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(int), 1, unit);
#endif
}


void int_array_getbin (FILE* unit, int *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    unsigned k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(int)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(int) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(int), nr, unit);
#endif	
}
    

/***********************************************************************
 * void uint_getbin (FILE* unit, unsigned int *var)
 * void uint_array_getbin (FILE* unit, unsigned int *var, int nr)
 *
 * read binary unsigned int (unsigned int array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/
void uint_getbin (FILE* unit, unsigned int *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
    int n;

#if defined(__LITTLE_ENDIAN__)
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(unsigned int)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(unsigned int), 1, unit);
#endif
}


void uint_array_getbin (FILE* unit, unsigned int *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    unsigned k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(unsigned int)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(unsigned int) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(unsigned int), nr, unit);
#endif	
}
    

/***********************************************************************
 * void short_getbin (FILE* unit, short *var)
 * void short_array_getbin (FILE* unit, short *var, int nr)
 *
 * read binary short (short array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/
void short_getbin (FILE* unit, short *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
    int n;

#if defined(__LITTLE_ENDIAN__)
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(short)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(short), 1, unit);
#endif
}


void short_array_getbin (FILE* unit, short *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    unsigned k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(short)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(short) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(short), nr, unit);
#endif	
}
    

/***********************************************************************
 * void ushort_getbin (FILE* unit, unsigned short *var)
 * void ushort_array_getbin (FILE* unit, unsigned short *var, int nr)
 *
 * read binary unsigned short (unsigned short array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void ushort_getbin (FILE* unit, unsigned short *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
    int n;

#if defined(__LITTLE_ENDIAN__)
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(unsigned short)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(unsigned short), 1, unit);
#endif
}


void ushort_array_getbin (FILE* unit, unsigned short *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    unsigned k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(unsigned short)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(unsigned short) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(unsigned short), nr, unit);
#endif	
}
    

/***********************************************************************
 * void long_getbin (FILE* unit, long *var)
 * void long_array_getbin (FILE* unit, long *var, int nr)
 *
 * read binary long (long array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void long_getbin (FILE* unit, long *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
    int n;

#if defined(__LITTLE_ENDIAN__)
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(long)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(long), 1, unit);
#endif
}


void long_array_getbin (FILE* unit, long *var, int nr)
{ 
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    unsigned k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(long)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(long) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(long), nr, unit);
#endif	
}
    
/***********************************************************************
 * void ulong_getbin (FILE* unit, unsigned long *var)
 * void ulong_array_getbin (FILE* unit, unsigned long *var, int nr)
 *
 * read binary unsigned long (unsigned long array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void ulong_getbin (FILE* unit, unsigned long *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
    int n;

#if defined(__LITTLE_ENDIAN__)
/* Linux binary reading (byte swapping : GOME-lv2 is SUN binary */
    for ( n = sizeof(unsigned long)-1; n>=0; n-- )
    {
	size = fread(vo + n , sizeof(char), 1, unit);
    }
#else    
    size = fread (vo, sizeof(unsigned long), 1, unit);
#endif
}


void ulong_array_getbin (FILE* unit, unsigned long *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
#if defined(__LITTLE_ENDIAN__)
    unsigned k;
    int n;
    for (k=0; k < nr; k++)
    {
	for ( n=sizeof(unsigned long)-1; n>=0; n-- )
	    size = fread(vo + k*sizeof(unsigned long) + n , sizeof(char), 1, unit);
    }
#else
    size = fread (vo, sizeof(unsigned long), nr, unit);
#endif	
}
    


/***********************************************************************
 * void char_getbin (FILE* unit, char *var)
 * void char_array_getbin (FILE* unit, char *var, int nr)
 *
 * read binary char (char array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void char_getbin (FILE* unit, char *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

    size = fread (vo, sizeof(char), 1, unit);
}


void char_array_getbin (FILE* unit, char *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
    size = fread (vo, sizeof(char), nr, unit);
}
    
/***********************************************************************
 * void uchar_getbin (FILE* unit, unsigned char *var)
 * void uchar_array_getbin (FILE* unit, unsigned char *var, int nr)
 *
 * read binary unsigned char (unsigned char array) from file 
 *  Depending from host system, swap bytes
 *
 ***********************************************************************/

void uchar_getbin (FILE* unit, unsigned char *var)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;

    size = fread (vo, sizeof(unsigned char), 1, unit);
}


void uchar_array_getbin (FILE* unit, unsigned char *var, int nr)
{
    size_t size;
    unsigned char* vo = (unsigned char*) var;
    size = fread (vo, sizeof(unsigned char), nr, unit);
}
    
/**********************************************************************
 * getbin routines for composed types, calling the above routines for
 * simple types
 *
 **********************************************************************/

void MJD_getbin (FILE* unit, MJD *var)
{
  int_getbin(unit, &var->days);
  uint_getbin(unit, &var->secnd);
  uint_getbin(unit, &var->musec);
}


void MJD_array_getbin (FILE* unit, MJD *var, int nr)
{
  int n;
  for (n=0; n<nr; n++)
    {
      int_getbin(unit, &var[n].days);
      uint_getbin(unit, &var[n].secnd);
      uint_getbin(unit, &var[n].musec);
    }
}

void GeoN_getbin (FILE* unit, GeoN *var)
{
  float_getbin (unit, &var->esm_pos);
  float_array_getbin (unit, var->sza_toa, 3);
  float_array_getbin (unit, var->saa_toa, 3);
  float_array_getbin (unit, var->los_zen, 3);
  float_array_getbin (unit, var->los_azi, 3);
  float_getbin (unit, &var->sat_height);
  float_getbin (unit, &var->earth_radius);
  Coord_getbin (unit, &var->sub_sat);
  Coord_array_getbin (unit, var->corner_coord, 4);
  Coord_getbin (unit, &var->centre_coord);
}
    
void GeoN_array_getbin (FILE* unit, GeoN *var, int nr)
{
    int n;
    for (n=0; n<nr; n++)
    {
      float_getbin (unit, &var[n].esm_pos);
      float_array_getbin (unit, var[n].sza_toa, 3);
      float_array_getbin (unit, var[n].saa_toa, 3);
      float_array_getbin (unit, var[n].los_zen, 3);
      float_array_getbin (unit, var[n].los_azi, 3);
      float_getbin (unit, &var[n].sat_height);
      float_getbin (unit, &var[n].earth_radius);
      Coord_getbin (unit, &var[n].sub_sat);
      Coord_array_getbin (unit, var[n].corner_coord, 4);
      Coord_getbin (unit, &var[n].centre_coord);
    }
}


void GeoL_getbin (FILE* unit, GeoL *var)
{
    float_getbin (unit, &var->esm_pos);
    float_getbin (unit, &var->asm_pos);
    float_array_getbin (unit, var->sza_toa, 3);
    float_array_getbin (unit, var->saa_toa, 3);
    float_array_getbin (unit, var->los_zen, 3);
    float_array_getbin (unit, var->los_azi, 3);
    float_getbin (unit, &var->sat_height);
    float_getbin (unit, &var->earth_radius);
    Coord_getbin (unit, &var->sub_sat);
    Coord_array_getbin (unit, var->tangent_ground_point, 3);
    float_array_getbin (unit, var->tangent_height, 3);
    float_getbin (unit, &var->doppler_shift);
}
    
void GeoL_array_getbin (FILE* unit, GeoL *var, int nr)
{
    int n;
    for (n=0; n<nr; n++)
    {
	float_getbin (unit, &var[n].esm_pos);
	float_getbin (unit, &var[n].asm_pos);
	float_array_getbin (unit, var[n].sza_toa, 3);
	float_array_getbin (unit, var[n].saa_toa, 3);
	float_array_getbin (unit, var[n].los_zen, 3);
	float_array_getbin (unit, var[n].los_azi, 3);
	float_getbin (unit, &var[n].sat_height);
	float_getbin (unit, &var[n].earth_radius);
	Coord_getbin (unit, &var[n].sub_sat);
	Coord_array_getbin (unit, var[n].tangent_ground_point, 3);
	float_array_getbin (unit, var[n].tangent_height, 3);
	float_getbin (unit, &var[n].doppler_shift);
    }
}

void GeoCal_getbin (FILE* unit, GeoCal *var)
{
    float_getbin (unit, &var->esm_pos);
    float_getbin (unit, &var->asm_pos);
    float_getbin (unit, &var->sza);
    Coord_getbin (unit, &var->sub_sat);
}

void GeoCal_array_getbin (FILE* unit, GeoCal *var, int nr)
{
    int n;
    for (n=0; n<nr; n++)
    {
	float_getbin (unit, &var[n].esm_pos);
	float_getbin (unit, &var[n].asm_pos);
	float_getbin (unit, &var[n].sza);
	Coord_getbin (unit, &var[n].sub_sat);
    }
}


void Coord_getbin (FILE* unit, Coord *var)
{
  int_getbin (unit, &var->lat);
  int_getbin (unit, &var->lon);
}


void Coord_array_getbin (FILE* unit, Coord *var, int nr)
{
  int n;
  for (n=0; n<nr; n++)
    {
      int_getbin (unit, &var[n].lat);
      int_getbin (unit, &var[n].lon);
    }
}



void cal_options_GADS_getbin (FILE* unit, cal_options_GADS *var)
{
    char_array_getbin ( unit, var->l1b_product_name, 62);
    char_getbin ( unit, &var-> geo_filter_flag );
    float_getbin ( unit, &var-> start_lat );
    float_getbin ( unit, &var-> start_lon );
    float_getbin ( unit, &var-> end_lat);   
    float_getbin ( unit, &var-> end_lon);   
    char_getbin ( unit, &var->  time_filter_flag ); 
    MJD_getbin ( unit, &var->  start_time);            
    MJD_getbin ( unit, &var->  stop_time );            
    char_getbin ( unit, &var->  category_filter_flag  );
    ushort_array_getbin ( unit, var->  category, 5 );      
    char_getbin ( unit, &var-> nadir_mds_flag  );       
    char_getbin ( unit, &var-> limb_mds_flag); 
    char_getbin ( unit, &var-> occ_mds_flag ); 
    char_getbin ( unit, &var-> mon_mds_flag ); 
    char_getbin ( unit, &var-> pmd_mds_flag ); 
    char_getbin ( unit, &var-> frac_pol_mds_flag  );
    char_getbin ( unit, &var-> slit_function_gads_flag  );
    char_getbin ( unit, &var-> sun_mean_ref_gads_flag); 
    char_getbin ( unit, &var-> leakage_current_gads_flag);
    char_getbin ( unit, &var-> spectral_cal_gads_flag); 
    char_getbin ( unit, &var-> pol_sens_gads_flag ); 
    char_getbin ( unit, &var-> rad_sens_gads_flag ); 
    char_getbin ( unit, &var-> ppg_etalon_gads_flag  ); 
    ushort_getbin ( unit, &var->  num_nadir_clusters );
    ushort_getbin ( unit, &var->  num_limb_clusters  );
    ushort_getbin ( unit, &var->  num_occ_clusters); 
    ushort_getbin ( unit, &var->  num_mon_clusters); 
    char_array_getbin ( unit, var->  nadir_cluster_flag  ,64  );
    char_array_getbin ( unit, var->  limb_cluster_flag,64); 
    char_array_getbin ( unit, var->  occ_cluster_flag ,64); 
    char_array_getbin ( unit, var->  mon_cluster_flag ,64); 
    char_getbin ( unit, &var->  mem_effect_cal_flag); 
    char_getbin ( unit, &var->  leakage_current_cal_flag );
    char_getbin ( unit, &var->  straylight_cal_flag); 
    char_getbin ( unit, &var->  ppg_cal_flag ); 
    char_getbin ( unit, &var->  etalon_cal_flag );
    char_getbin ( unit, &var->  spectral_cal_flag  );
    char_getbin ( unit, &var->  polarisation_cal_flag );
    char_getbin ( unit, &var->  radiance_cal_flag  );
}


void ads_states_getbin (FILE* unit, ads_states *var)
{
    MJD_getbin ( unit, &var-> StartTime);
    uchar_getbin ( unit, &var->  mds_attached);
    uchar_getbin ( unit, &var->  flag_reason);
    float_getbin ( unit, &var-> orbit_phase);
    ushort_getbin ( unit, &var-> category);
    ushort_getbin ( unit, &var-> state_id);
    ushort_getbin ( unit, &var-> duration);
    ushort_getbin ( unit, &var-> longest_int_time);
    ushort_getbin ( unit, &var-> num_clusters);
    Clcon_array_getbin ( unit, var-> Clcon, MAX_CLUSTER);
    uchar_getbin ( unit, &var->  flag_mds);
    ushort_getbin ( unit, &var-> num_aux);
    ushort_getbin ( unit, &var-> num_pmd);
    ushort_getbin ( unit, &var-> num_int);
    ushort_array_getbin ( unit, var-> int_times, MAX_CLUSTER);
    ushort_array_getbin ( unit, var-> num_polar, MAX_CLUSTER);
    ushort_getbin ( unit, &var-> total_polar);
    ushort_getbin ( unit, &var-> num_dsr);
    uint_getbin ( unit, &var->  length_dsr);

}


void Clcon_getbin (FILE* unit, Clcon *var)
{
      uchar_getbin ( unit, &var-> id);
      uchar_getbin ( unit, &var-> channel);
      ushort_getbin ( unit, &var-> pixel_nr);
      ushort_getbin ( unit, &var-> length);
      float_getbin ( unit, &var-> pet);
      ushort_getbin ( unit, &var-> int_time);
      ushort_getbin ( unit, &var-> coadd);
      ushort_getbin ( unit, &var-> n_read);
      uchar_getbin ( unit, &var-> type);
}





void Clcon_array_getbin (FILE* unit, Clcon *var, int nr)
{
    int n;
    for (n=0; n<nr; n++)
    {
	Clcon_getbin (unit, &var[n]);
    }
}



void gads_sun_ref_getbin (FILE* unit, gads_sun_ref *var)
{
    char_array_getbin ( unit, var-> id, 2);
    float_array_getbin ( unit, var-> wavel, NPIXEL);
    float_array_getbin ( unit, var-> spectrum, NPIXEL);
    float_array_getbin ( unit, var-> precision, NPIXEL);
    float_array_getbin ( unit, var-> accuracy, NPIXEL);
    float_array_getbin ( unit, var-> etalon, NPIXEL);
    float_getbin ( unit, &var-> asm_pos);
    float_getbin ( unit, &var-> esm_pos);
    float_getbin ( unit, &var-> sun_elev);
    float_array_getbin ( unit, var-> pmd_mean, NPMD);
    float_array_getbin ( unit, var-> out_of_band_nd_out, NPMD);
    float_array_getbin ( unit, var-> out_of_band_nd_in, NPMD);
    float_getbin ( unit, &var-> doppler);
}


void mds_1c_constant_getbin (FILE* unit, mds_1c_constant *var)
{
    MJD_getbin ( unit, &var-> StartTime);
    int_getbin ( unit, &var-> length);
    char_getbin ( unit, &var-> quality);
    float_getbin ( unit, &var-> orbit_phase);
    ushort_getbin ( unit, &var-> category);
    ushort_getbin ( unit, &var-> state_id);
    ushort_getbin ( unit, &var-> cluster_id);
    ushort_getbin ( unit, &var-> nobs);
    ushort_getbin ( unit, &var-> npixels);
    uchar_getbin ( unit, &var-> unit_flag);
}
