/*--------------------------------------------------------------------*\
** 
** Example program to read SCIAMACHY Level 1c nadir data
** 
** Author: S. Noel, IFE/IUP Uni Bremen 
**        (Stefan.Noel@iup.physik.uni.bremen.de)
** 
** Version: 0.1, 17 Apr 2002 (first beta release)
** 
\*--------------------------------------------------------------------*/

#include<stdlib.h>
#include<string.h>
#include<stdio.h>

#include "lv1c_struct.h" /* rprogram_nameequires "lv1_struct.h" ! */

#include "read1c_defs.h"
#include "read1c_funct.h"

#define PROGRAM_VERSION "0.55"

int main(int argc, char* argv[])
{
    char* FILE_name = argv[1]; 	/* first argument is filename */
    FILE* out;
    char outname[100];
    char sun_spect_id[3];

    float wlstart, wlend; 
/*     int pix_start, pix_end; */
    int i;
    int n_read_nadir = 0;
    int n_read_limb = 0;
    int n_read_mon = 0;
    int channel = 0;
    int n_arg;

    user_data ud;		/* This holds the read data */
    user_data_solar uds;
    
    info_l1c info;      /* this will contain all internal 
			   information like data offsets etc. */

    if (argc < 6) {
	printf("Program: %s, Version: %s\n\n", argv[0], PROGRAM_VERSION);
	printf("Extracts level 1c nadir and limb  data for specified wavelength window.\n"
	       "Optionally, specify a channel (1-8) to avoid output from other channels\n "
	       "because of the overlap regions. Even if the clusters are restricted in L1C,\n"
	       "the problem would occur in the solar reference spectrum.\n\n"
	       "The solar reference spectrum has to be selected.\n\n");
	printf("Usage : %s wl1 wl2 channel sun l1c_file [l1c_file_2 ...]\n",
	       argv[0]);
	printf(" 212 < wl1 < wl2 < 2386 \n"
	       " 1<=channel<=8,  channel = 0 means no channel restriction.\n" 
	       " sun : select solar reference spectrum:\n"
	       "       D1  : Sun reference over ESM Diffuser\n"
	       "       O   : Occultation sun reference\n"
	       "       S   : Sun reference over Subsolar Port\n"
	       "       D0  : in early products instead of D1 with a low resolution spectrum\n"
	       "       D2 .. D9 : in early products as dummy solar spectrum available (999.0 only)\n"
	       "  other IDs may occur, check with EnviView, if in trouble.\n"
	       "              (Sun Reference Spectrum GADS  -  sun_spect_id)\n"
               " n_read    : start output counter with n_read instead of 0\n\n");
/*	      
#if defined(__LITTLE_ENDIAN__)
	printf(" **** LITTLE ENDIAN (LINUX) VERSION **** \n");
#else
	printf(" **** BIG ENDIAN (SUN) VERSION **** \n");
#endif
	exit(1);
*/	
    }
				/* Read  command line */
    FILE_name = argv[1];
/*     pix_start = atoi (argv[2]); */
/*     pix_end = atoi (argv[3]); */
    sscanf (argv[1], "%f", &wlstart);
    sscanf (argv[2], "%f", &wlend);
    sscanf (argv[3], "%d", &channel);
				/*  Read sun_spec_id */
    strncpy (sun_spect_id, argv[4], 2);
    if (strlen (sun_spect_id) == 1)
	strcpy (sun_spect_id + 1, " ");

    for (n_arg = 5; n_arg<argc; n_arg++)
    {
	FILE_name = argv [n_arg];
				/* open l1c file */
    if ( openL1c(FILE_name, &info) != OK ) {    
	printf("*** ERROR opening file %s ****\n", FILE_name);
	exit(1);
    }
    printf("File successfully opened\n");
    
    /* set wl window : Limited use because of overlap regions */
    printf ("%8.2f %8.2f\n", wlstart, wlend);
    if (set_wl_window (&info, wlstart, wlend, channel) != OK )
    {
	printf("*** Setting WL window ***\n");
	exit(1);
	} 
    
/* 				/\* set pix window *\/ */
/*     if (set_pix_window (&info, pix_start, pix_end) != OK ) */
/*     { */
/* 	printf("*** Setting pix window %i - %i***\n",pix_start,pix_end); */
/* 	exit(1); */
/*     } */

    /* This part can be used for cluster setting */
    /* In this case, the solar spectrum covers all wavelength */
    /*
    int cluster_ids[] = {12,14};
    int n_cluster_id = 2;
    if ( set_clusters (&info, cluster_ids, n_cluster_id) != OK )
    {
	printf("*** Setting clusters error ***\n");
	exit(1);
    } 
    */
           
				/* read solar spectrum */

    if (read_solar (&info, sun_spect_id, &uds))
    {
	printf("*** ERROR reading solar spectrum ***\n");
	exit(1);
    }
       	
				/* output solar spectrum*/

    out = fopen("SCIA_solar.dat", "w");
    
    fprintf (out, "L1b product        : %s\n", uds.l1b_product_name);
    fprintf (out, "Orbit nr.          : %5d\n", uds.abs_orbit);
    fprintf (out, "Ver. Proc.Soft.    : %s\n"
	     "Ver. Keydata       : %s\n"
	     "Ver. M-factor      : %s\n",
	     uds.software_ver,
	     uds.key_data_version,
	     uds.m_factor_version);
    fprintf (out, "Calibr. appl. (0-8): %s\n", uds.cal_applied);
    fprintf (out, "Solar spectrum ID : %2s\n\n" , uds.sun_spect_id);

    fprintf (out, "\nNumber of pixels: %5i\n",uds.n_wl);    
    fprintf (out, "pix chan wl[nm]  irrad[..]  irrad_err[..]\n");
    for ( i=0; i < uds.n_wl; i++)
    {
	fprintf(out,"%4d %1d   %7.2f %15.8g %15.8g\n",
		uds.pixel_nr[i],
		uds.pixel_ch[i],
		uds.wl[i],
		uds.signal[i],
		uds.signal_err[i]);
    }
    
    fclose (out);
				/* free memory allocated in read_solar */
    free_user_data_solar (&uds);
				/* Loop over all nadir pixel
				    read_next_nadir: reads one bye one
				    Don't worry about states :-) */

    while (read_next_mds (&info, &ud, NADIR) == OK)
    {
				/* output to file for each nadir measurement */
	sprintf (outname,"SCIA_nadir_%05i.dat",n_read_nadir);
	printf ("###### %s\n", outname);
	out = fopen(outname, "w");
	fprintf (out, "L1b product        : %s\n", ud.l1b_product_name);
	fprintf (out, "Orbit nr.          : %5d\n", ud.abs_orbit);
	fprintf (out, "Ver. Proc.Soft.    : %s\n"
		      "Ver. Keydata       : %s\n"
		      "Ver. M-factor      : %s\n",
		 ud.software_ver,
		 ud.key_data_version,
		 ud.m_factor_version);
	fprintf (out, "Calibr. appl. (0-8): %s\n", ud.cal_applied);
	fprintf (out, "State ID           : %d\n", ud.state_id);
	fprintf (out, "State Starttime    : %s\n", ud.state_date);
	fprintf (out, "Nr. state/readout  : %3d %3d\n\n",
		 ud.n_state, ud.n_readout);
	fprintf (out, "Starttime  : %s\n", ud.date);
	fprintf (out, "Int. time  :%11.5f\n", ud.int_time);
	fprintf (out, "Sat. Height:%11.5f\n", ud.geo_nadir.sat_height);
	fprintf (out, "Earth rad. :%11.5f\n", ud.geo_nadir.earth_radius);
	
	fprintf (out, "Center     :%8.2f %8.2f\n",
		 ((double) ud.geo_nadir.centre_coord.lat) * 1e-6,
		 ((double) ud.geo_nadir.centre_coord.lon) * 1e-6);
	fprintf (out, "SZA(TOA)   :%8.2f %8.2f %8.2f\n",
		 ud.geo_nadir.sza_toa[0],
		 ud.geo_nadir.sza_toa[1],
		 ud.geo_nadir.sza_toa[2]);
	fprintf (out, "SAA(TOA)   :%8.2f %8.2f %8.2f\n",
		 ud.geo_nadir.saa_toa[0],
		 ud.geo_nadir.saa_toa[1],
		 ud.geo_nadir.saa_toa[2]);
	fprintf (out, "LOS(Zen)   :%8.2f %8.2f %8.2f\n",
		 ud.geo_nadir.los_zen[0],
		 ud.geo_nadir.los_zen[1],
		 ud.geo_nadir.los_zen[2]);
	fprintf (out, "LOS(Azi)   :%8.2f %8.2f %8.2f\n",
		 ud.geo_nadir.los_azi[0],
		 ud.geo_nadir.los_azi[1],
		 ud.geo_nadir.los_azi[2]);
	fprintf (out, "Corners    :%8.2f %8.2f  %8.2f %8.2f  "
		 "%8.2f %8.2f  %8.2f %8.2f\n",
		 ((double) ud.geo_nadir.corner_coord[0].lat * 1e-6),
		 ((double) ud.geo_nadir.corner_coord[0].lon * 1e-6),
		 ((double) ud.geo_nadir.corner_coord[1].lat * 1e-6),
		 ((double) ud.geo_nadir.corner_coord[1].lon * 1e-6),
		 ((double) ud.geo_nadir.corner_coord[2].lat * 1e-6),
		 ((double) ud.geo_nadir.corner_coord[2].lon * 1e-6),
		 ((double) ud.geo_nadir.corner_coord[3].lat * 1e-6),
		 ((double) ud.geo_nadir.corner_coord[3].lon * 1e-6));
				/*  PMDs if available */
	if (ud.n_pmd > 0)
	{
	    fprintf (out, "\nPMDS : %3i\n", ud.n_pmd);
	    for (i=0; i< ud.n_pmd; i++)
		fprintf (out, "%8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n",
			 ud.pmd[i].pmd[0],
			 ud.pmd[i].pmd[1],
			 ud.pmd[i].pmd[2],
			 ud.pmd[i].pmd[3],
			 ud.pmd[i].pmd[4],
			 ud.pmd[i].pmd[5],
			 ud.pmd[i].pmd[6]);
	}
				/*  Wavelength window */
	fprintf (out, "\nNumber of pixels: %5i\n",ud.n_wl);
	fprintf (out, "pix chan clus coad  wl[nm] wl_err[nm] rad[..]    rad_err[..]\n");
	for (i=0; i<ud.n_wl; i++)
	{
	    fprintf (out, "%4i  %2i %2i  %2i   %8.2f %8.2f  %12.6e %12.6e\n",
		     ud.pixel_nr[i],
		     ud.pixel_ch[i],
		     ud.pixel_cls[i],
		     ud.pixel_coadd[i],
		     ud.wl[i],
		     ud.wl_err[i],
		     ud.signal[i],
		     ud.signal_err[i]);
	}
	n_read_nadir++;
				/* Don't forget to free memory
				   before reading next dataset!
				   Otherwise their is a memory hole */
	free_user_data (&ud);
	
	fclose (out);
    }
    
    printf ("%i Nadir datasets read\n", n_read_nadir);
				/* close l1c file */
    closeL1c (&info);

    /* ******************* Try limb readout here, at the moment only
     one type of readout per open statement is allowed */

    if ( openL1c(FILE_name, &info) != OK ) {
	printf("*** ERROR opening file %s ****\n", FILE_name);
	exit(1);
   }
    printf("File successfully opened\n");

    /* set wl window : Overlap regions are double */
    printf ("%8.2f %8.2f\n", wlstart, wlend);
    if (set_wl_window (&info, wlstart, wlend, channel) != OK )
    {
	printf("*** Setting WL window ***\n");
	exit(1);
    }

    while (read_next_mds (&info, &ud, LIMB) == OK)
    {
				/* output to file for each nadir measurement */
	sprintf (outname,"SCIA_limb_%05i.dat",n_read_limb);
	printf ("###### %s\n", outname);
	out = fopen(outname, "w");
	fprintf (out, "L1b product        : %s\n", ud.l1b_product_name);
	fprintf (out, "Orbit nr.          : %5d\n", ud.abs_orbit);
	fprintf (out, "Ver. Proc.Soft.    : %s\n"
		      "Ver. Keydata       : %s\n"
		      "Ver. M-factor      : %s\n",
		 ud.software_ver,
		 ud.key_data_version,
		 ud.m_factor_version);
	fprintf (out, "Calibr. appl. (0-8): %s\n", ud.cal_applied);
	fprintf (out, "State ID           : %d\n", ud.state_id);
	fprintf (out, "State Starttime    : %s\n", ud.state_date);
	fprintf (out, "Nr. state/readout  : %3d %3d\n\n",
		 ud.n_state, ud.n_readout);
	fprintf (out, "Starttime  : %s\n", ud.date);
	fprintf (out, "Int. time  :%11.5f\n", ud.int_time);
	fprintf (out, "Sat. Height:%11.5f\n", ud.geo_limb.sat_height);
	fprintf (out, "Earth rad. :%11.5f\n", ud.geo_limb.earth_radius);

	fprintf (out, "Tangent_ground_points :%8.2f %8.2f  "
		 "%8.2f %8.2f  %8.2f %8.2f\n",
		 ((double) ud.geo_limb.tangent_ground_point[0].lat) * 1e-6,
		 ((double) ud.geo_limb.tangent_ground_point[0].lon) * 1e-6,
		 ((double) ud.geo_limb.tangent_ground_point[1].lat) * 1e-6,
		 ((double) ud.geo_limb.tangent_ground_point[1].lon) * 1e-6,
		 ((double) ud.geo_limb.tangent_ground_point[2].lat) * 1e-6,
		 ((double) ud.geo_limb.tangent_ground_point[2].lon) * 1e-6);
	fprintf (out, "Tangent_height        :%8.2f %8.2f %8.2f\n",
		 ud.geo_limb.tangent_height[0],
		 ud.geo_limb.tangent_height[1],
		 ud.geo_limb.tangent_height[2]);
	fprintf (out, "SZA(TOA)              :%8.2f %8.2f %8.2f\n",
		 ud.geo_limb.sza_toa[0],
		 ud.geo_limb.sza_toa[1],
		 ud.geo_limb.sza_toa[2]);
	fprintf (out, "SAA(TOA)              :%8.2f %8.2f %8.2f\n",
		 ud.geo_limb.saa_toa[0],
		 ud.geo_limb.saa_toa[1],
		 ud.geo_limb.saa_toa[2]);
	fprintf (out, "LOS(Zen)              :%8.2f %8.2f %8.2f\n",
		 ud.geo_limb.los_zen[0],
		 ud.geo_limb.los_zen[1],
		 ud.geo_limb.los_zen[2]);
	fprintf (out, "LOS(Azi)              :%8.2f %8.2f %8.2f\n",
		 ud.geo_limb.los_azi[0],
		 ud.geo_limb.los_azi[1],
		 ud.geo_limb.los_azi[2]);
	fprintf (out, "Sub_Sat               :%8.2f %8.2f\n",
		 ((double) ud.geo_limb.sub_sat.lat) * 1e-6,
		 ((double) ud.geo_limb.sub_sat.lon) * 1e-6);	    
	fprintf (out, "Doppler shift         :%8.2f\n",
		 ud.geo_limb.doppler_shift);
				/*  PMDs if available */
	if (ud.n_pmd > 0)
	{
	    fprintf (out, "PMDS : %3i\n", ud.n_pmd);
	    for (i=0; i< ud.n_pmd; i++)
		fprintf (out, "%8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n",
			 ud.pmd[i].pmd[0],
			 ud.pmd[i].pmd[1],
			 ud.pmd[i].pmd[2],
			 ud.pmd[i].pmd[3],
			 ud.pmd[i].pmd[4],
			 ud.pmd[i].pmd[5],
			 ud.pmd[i].pmd[6]);
	}
				/*  Wavelength window */
	fprintf (out, "\nNumber of pixels: %5i\n",ud.n_wl);
	fprintf (out, "pix chan clus coad  wl[nm] wl_err[nm] rad[..]    rad_err[..]\n");
	for (i=0; i<ud.n_wl; i++)
	{
	    fprintf (out, "%4i  %2i %2i  %2i   %8.2f %8.2f  %12.6e %12.6e\n",
		     ud.pixel_nr[i],
		     ud.pixel_ch[i],
		     ud.pixel_cls[i],
		     ud.pixel_coadd[i],
		     ud.wl[i],
		     ud.wl_err[i],
		     ud.signal[i],
		     ud.signal_err[i]);
	}
	n_read_limb++;
				/* Don't forget to free memory
				   before reading next dataset!
				   Otherwise their is a memory hole */
	free_user_data (&ud);
	
	fclose (out);
    }
    printf ("%i Limb datasets read\n", n_read_limb);
				/* close l1c file */
    closeL1c (&info);

    /* ******************* Try Monitoring readout here, at the moment only
     one type of readout per open statement is allowed */

    if ( openL1c(FILE_name, &info) != OK ) {
	printf("*** ERROR opening file %s ****\n", FILE_name);
	exit(1);
   }
    printf("File successfully opened\n");

    /* set wl window : Overlap regions are double */
    printf ("%8.2f %8.2f\n", wlstart, wlend);
    if (set_wl_window (&info, wlstart, wlend, channel) != OK )
    {
	printf("*** Setting WL window ***\n");
	exit(1);
    }

    while (read_next_mds (&info, &ud, MONITORING) == OK)
    {
				/* output to file for each nadir measurement */
	sprintf (outname,"SCIA_mon_%05i.dat",n_read_mon);
	printf ("###### %s\n", outname);
	out = fopen(outname, "w");
	fprintf (out, "L1b product        : %s\n", ud.l1b_product_name);
	fprintf (out, "Orbit nr.          : %5d\n", ud.abs_orbit);
	fprintf (out, "Ver. Proc.Soft.    : %s\n"
		      "Ver. Keydata       : %s\n"
		      "Ver. M-factor      : %s\n",
		 ud.software_ver,
		 ud.key_data_version,
		 ud.m_factor_version);
	fprintf (out, "Calibr. appl. (0-8): %s\n", ud.cal_applied);
	fprintf (out, "State ID           : %d\n", ud.state_id);
	fprintf (out, "State Starttime    : %s\n", ud.state_date);
	fprintf (out, "Nr. state/readout  : %3d %3d\n\n",
		 ud.n_state, ud.n_readout);
	fprintf (out, "Starttime  : %s\n", ud.date);
	fprintf (out, "Int. time  :%11.5f\n", ud.int_time);
	fprintf (out, "Sub_Sat               :%8.2f %8.2f\n",
		 ((double) ud.geo_cal.sub_sat.lat) * 1e-6,
		 ((double) ud.geo_cal.sub_sat.lon) * 1e-6);	    
	fprintf (out, "SZA                   :%8.2f\n",
		 ud.geo_cal.sza);
	fprintf (out, "ESM Pos.              :%8.2f\n",
		 ud.geo_cal.esm_pos);
	fprintf (out, "ASM_Pos.              :%8.2f\n",
		 ud.geo_cal.asm_pos);
				/*  PMDs if available */
	if (ud.n_pmd > 0)
	{
	    fprintf (out, "PMDS : %3i\n", ud.n_pmd);
	    for (i=0; i< ud.n_pmd; i++)
		fprintf (out, "%8.2f %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n",
			 ud.pmd[i].pmd[0],
			 ud.pmd[i].pmd[1],
			 ud.pmd[i].pmd[2],
			 ud.pmd[i].pmd[3],
			 ud.pmd[i].pmd[4],
			 ud.pmd[i].pmd[5],
			 ud.pmd[i].pmd[6]);
	}
				/*  Wavelength window */
	fprintf (out, "\nNumber of pixels: %5i\n",ud.n_wl);
	fprintf (out, "pix chan clus coad  wl[nm] wl_err[nm] rad[..]    rad_err[..]\n");
	for (i=0; i<ud.n_wl; i++)
	{
	    fprintf (out, "%4i  %2i %2i  %2i   %8.2f %8.2f  %12.6e %12.6e\n",
		     ud.pixel_nr[i],
		     ud.pixel_ch[i],
		     ud.pixel_cls[i],
		     ud.pixel_coadd[i],
		     ud.wl[i],
		     ud.wl_err[i],
		     ud.signal[i],
		     ud.signal_err[i]);
	}
	n_read_mon++;
				/* Don't forget to free memory
				   before reading next dataset!
				   Otherwise their is a memory hole */
	free_user_data (&ud);
	
	fclose (out);
    }
    printf ("%i Monitoring datasets read\n", n_read_mon);
				/* close l1c file */
    closeL1c (&info);

	

    }
    return 0;
}






