/*--------------------------------------------------------------------*\
** 
** Routines to read SCIAMACHY Level 1c product
** 
** Author: S. Noel, IFE/IUP Uni Bremen 
**        (Stefan.Noel@iup.physik.uni.bremen.de)
** 
** Version: 0.1, 17 Apr 2002 (first beta release)
** 
\*--------------------------------------------------------------------*/

#include<stdlib.h>
#include<stdio.h>
#include<string.h>
#include<math.h>

#include "lv1_defs.h"
#include "lv1_struct.h"
#include "lv1c_struct.h" /* requires "lv1_struct.h" ! */

#include "UTC_string.h"

#include "read1c_defs.h"
#include "read1c_funct.h"
#include "bin_read.h"
#include "UTC_string.h"



L1C_error read_next_pmd_state (info_l1c *info, L1C_MDS_type type);
unsigned long Read_PMD_MDS (FILE* unit,
			    long int offset,
			    nadir_pmd *pmd, L1C_MDS_type type);


/* codes for spectra; these need to be synchronised with solar_type definitions */
/* const char *const sun_ref_code[] = {"D1", "O ", "S ", "D2", "D3",
                                    "D4", "D5", "D6","D7", "D8", "D9",
                                    "D0"};
*/

/*********************************************************************\
* Open L1c, specify what to read 
* reads all DSDs and stores read values in info structure
\*********************************************************************/

L1C_error openL1c (char* FILE_name, info_l1c *info)
{
    int num_dsd = 0;
    int i, cat;
    DSD dsd;
  
    info->FILE_l1c = fopen(FILE_name, "rb");
    if (info->FILE_l1c == NULL) {
	printf("Input file %s could not be opened! Abort!\n", FILE_name);
	return FILE_NOT_FOUND;
    }

    Read_MPH (info->FILE_l1c, &info->mph);
    Read_SPH (info->FILE_l1c, &info->sph);
#if defined(__DEBUG_L1C__)
    printf ("%s\n%s\n\n",info->mph.product, info->mph.spare_1);
#endif
    /*--------------------------------------------------------------------*\
    ** read MPH
    ** get number of DSDs 
    \*--------------------------------------------------------------------*/

  
    sscanf(info->mph.num_dsd, "%d", &num_dsd);
    info->num_dsd = num_dsd;


    /*--------------------------------------------------------------------*\
    ** read DSDs
    note: num_dsd seems to be too high by 1 (spare entry)! 
    \*--------------------------------------------------------------------*/

    for (i=0; i<info->num_dsd-1; i++)
    {
	Read_DSD ( info->FILE_l1c, &dsd );
          
#if defined(__DEBUG_L1C__)
	printf ("%s\n%c\n%s\n%i\n%i\n%i\n%i\n",
		dsd.name,
		dsd.type,
		dsd.filename,
		dsd.offset,
		dsd.size,
		dsd.num_dsr,
		dsd.size);
#endif
	/* check for dsd names */
	if (strcmp(dsd.name, "STATES") == 0) {
	    info->states = dsd;
	} 
	else if (strcmp(dsd.name, "SUN_REFERENCE") == 0) {
	    info->sun_ref = dsd;
	} 
	else if (strcmp(dsd.name, "NADIR_PMD") == 0) {
	    info->nadir_pmd = dsd;
	} 
	else if (strcmp(dsd.name, "LIMB_PMD") == 0) {
	    info->limb_pmd = dsd;
	} 
	else if (strcmp(dsd.name, "OCCULTATION_PMD") == 0) {
	    info->occ_pmd = dsd;
	} 
	else if (strcmp(dsd.name, "NADIR_FRAC_POL") == 0) {
	    info->nadir_pol = dsd;
	} 
	else if (strcmp(dsd.name, "LIMB_FRAC_POL") == 0) {
	    info->limb_pol = dsd;
	} 
	else if (strcmp(dsd.name, "OCCULTATION_FRAC_POL") == 0) {
	    info->occ_pol = dsd;
	} 
	else if (strcmp(dsd.name, "NADIR") == 0) {
	    info->nadir = dsd;
	} 
	else if (strcmp(dsd.name, "LIMB") == 0) {
	    info->limb = dsd;
	} 
	else if (strcmp(dsd.name, "OCCULTATION") == 0) {
	    info->occ = dsd;
	} 
	else if (strcmp(dsd.name, "MONITORING") == 0) {
	    info->mon = dsd;
	} 
	else if (strcmp(dsd.name, "CAL_OPTIONS") == 0) {
	    info->cal_options = dsd;
	} 
	else {
	    /* do nothing */
	}
    }
#if defined(__DEBUG_L1C__)
    printf ("open_lv1c");
    printf ("NADIR DSD %i\n",info->nadir.num_dsr);
#endif
    /* Put MDS offsets in array */
    info->mds_offset[NADIR] = info->nadir.offset;
    info->mds_offset[LIMB] = info->limb.offset;
    info->mds_offset[OCCULTATION] = info->occ.offset;
    info->mds_offset[MONITORING] = info->mon.offset;
    /* Put MDS num_dsr in array */
    info->n_mds[NADIR] = info->nadir.num_dsr;
    info->n_mds[LIMB] = info->limb.num_dsr;
    info->n_mds[OCCULTATION] = info->occ.num_dsr;
    info->n_mds[MONITORING] = info->mon.num_dsr;
    /* Put MDS num_dsr in array */
    info->n_pmd_mds[NADIR]       = info->nadir_pmd.num_dsr;
    info->n_pmd_mds[LIMB]        = info->limb_pmd.num_dsr;
    info->n_pmd_mds[OCCULTATION] = info->occ_pmd.num_dsr;
    info->n_pmd_mds[MONITORING]  = 0;
    /* Put MDS num_dsr in array */
    info->mds_pmd_offset[NADIR]       = info->nadir_pmd.offset;
    info->mds_pmd_offset[LIMB]        = info->limb_pmd.offset;
    info->mds_pmd_offset[OCCULTATION] = info->occ_pmd.offset;
    info->mds_pmd_offset[MONITORING]  = 0;
    
    /*  Read Calibr. Options */
    Read_cal_options_GADS (info);
    
    /*  fill in general user_file_info */
    info->user_file_info.product = info->mph.product;
    info->user_file_info.software_ver = info->mph.software_ver;
    info->user_file_info.abs_orbit = atoi (info->mph.abs_orbit);
    info->user_file_info.key_data_version = info->sph.key_data_version;
    info->user_file_info.m_factor_version = info->sph.m_factor_version;
    info->user_file_info.l1b_product_name = info->cal_options_GADS.l1b_product_name;
    i = 0;
    *info->user_file_info.cal_applied = '\0';
    if ( info->cal_options_GADS.mem_effect_cal_flag== -1 )
    {
	sprintf ( info->user_file_info.cal_applied + 2*i , " 0");
	i++;
    }
    if ( info->cal_options_GADS.leakage_current_cal_flag == -1 )
    {
	sprintf ( info->user_file_info.cal_applied + 2*i , " 1");
	i++;
    }
    if ( info->cal_options_GADS.straylight_cal_flag== -1 ) 
    {
	sprintf ( info->user_file_info.cal_applied + 2*i , " 2");
	i++;
    }
    if ( info->cal_options_GADS.ppg_cal_flag == -1 ) 
    {
	sprintf ( info->user_file_info.cal_applied + 2*i , " 3");
	i++;
    }
    if ( info->cal_options_GADS.etalon_cal_flag == -1 ) 
    {
	sprintf ( info->user_file_info.cal_applied + 2*i , " 4");
	i++;
    }
    if ( info->cal_options_GADS.spectral_cal_flag  == -1 )
    {
	sprintf ( info->user_file_info.cal_applied + 2*i , " 5");
	i++;
    }
    if ( info->cal_options_GADS.polarisation_cal_flag == -1 )
    {
	sprintf ( info->user_file_info.cal_applied + 2*i , " 6");
	i++;
    }
    if ( info->cal_options_GADS.radiance_cal_flag  == -1 ) 
    {
	sprintf ( info->user_file_info.cal_applied + 2*i , " 7");
	i++;
    }

/* Read ADS states  */
    if ( ADS_state (info) != OK )
	return NO_ADS_STATES;	
/* Count really available states */
#if defined(__DEBUG_L1C__)
    printf ("open_lv1c\n"
	    "Num nadir states: %i"
	    "\n Num limb states: %i\n", 
	    info->mds_offset[NADIR], info->mds_offset[LIMB]);
#endif
/* Build index lists for MDS <> states */
/* Trust that both are in consistent order */
/*  initialize arrays */
    for (cat = 0 ; cat < MAX_MDS_TYPES ; cat++)
    {
	info->idx_states[cat] = malloc (sizeof(int) *
					info->states.num_dsr);
	info->n_states[cat] = 0;
    }
/*  go through all states*/
    for (i=0; i < info->states.num_dsr; i++)
    {
	switch (info->ads_states[i].category)
	{
	    /*  count up categories and store index */
	    case 1:
	    case 3:
	    case 17:
	    case 18:
		info->idx_states[NADIR][info->n_states[NADIR]++] = i;
		break;	      
	    case 2:
		info->idx_states[LIMB][info->n_states[LIMB]++] = i;
		break;
	    case 4:
	    case 5:
	    case 6:
		info->idx_states[OCCULTATION][info->n_states[OCCULTATION]++] = i;
		break;
	    case 7:
	    case 8:
	    case 9:
	    case 10:
	    case 11:
	    case 12:
	    case 13:
	    case 14:
	    case 15:
	    case 16:
		info->idx_states[MONITORING][info->n_states[MONITORING]++] = i;
		break;
	    default:
#if defined(__DEBUG_L1C__)
		printf ("open_lv1c\n"
			" Measurement cat  %i not implemented yet ", cat);
#endif
		;
	}      
    }
/*  check if corresponding MDS are available */
/*  if not, simply set n_state[type] to zero */
    for (cat = 0 ; cat < MAX_MDS_TYPES ; cat++)
    {
	if ( info->n_mds[cat] == 0 )
	    info->n_states[cat] = 0;
    }
  
/* Read solar spektrum for wl-grid */  
    Fixed_Wavelength_grid (info);
/* set various counters to zero */
    info->cur_readout_in_state = 0;   
    info->cur_max_readout_in_state = 0;
    info->cur_pix_start = 0;           
    info->cur_pix_end = 0;
    info->n_wanted_clusters = 0;
    
    for (i=0; i<MAX_MDS_TYPES; i++)
	info->cur_state_nr[i] = 0;
    return OK;
}

  
L1C_error Fixed_Wavelength_grid (info_l1c *info)
{
    int n;
    /* use the pre-launch wavelength grid, stored in wvlen_det_pix.c
       in one large array */
#include "wvlen_det_pix.c"  
    for (n=0; n<NPIXEL; n++)
	info->wl[n] = wvlen_det_pix[n];
    return OK;
}

/*********************************************************************\
 * Define pixel window from wavelength window
 * kb 23.04.01
 * channel : specify you want to read (1 - 8) to avoid problems in
 *           overlap regions (data from other channel)
 *           channel=0  : Do not restrict channel
\*********************************************************************/

L1C_error set_wl_window (info_l1c *info, float wl_start, float wl_end,
			 int channel)
{
    int n;
    int search_start=0;
    int search_end=NPIXEL;
#if defined(__DEBUG_L1C__)    
    printf ("%8.2f %8.2f\n",wl_start,wl_end);
#endif
    if (channel > 0 && channel < 7 )
    {
	search_start = (channel-1) * 1024;
	search_end = channel * 1024;
    }

    
    for (n=search_start ; n<search_end; n++)
    {
	if (info->wl[n] > wl_start)
	    break;
    }
    if ( n == search_end )
	return WL_OUT_OF_RANGE;
    if ( n > search_start)
	info->cur_pix_start = n-1;
    else
	info->cur_pix_start = n;
    
    for (n = search_end; n>search_start; n--)
    {
	if ( info->wl[n-1] < wl_end )
	    break;
    }
    if (n == 0)
	return WL_OUT_OF_RANGE;
    info->cur_pix_end = n;

    if (info->cur_pix_start >= info->cur_pix_end)
	return WL_OUT_OF_RANGE;
    return OK;
}


/*********************************************************************\
 * Define pixel window from pixel_nr
 * kb 23.04.01
\*********************************************************************/

L1C_error set_pix_window (info_l1c *info, int pix_start, int pix_end)
{
    if (pix_start < 0 || pix_end > 8192 || pix_start >= pix_end)
	return  WL_OUT_OF_RANGE;
    info->cur_pix_end = pix_end;
    info->cur_pix_start = pix_start;
    return OK;
}


/*********************************************************************\
 * Define cluster instead of pixel windows from cluster list
 *  in integer array cluster_id  with n_cluster elements
 *       
 * kb 31.09.03
\*********************************************************************/

L1C_error set_clusters (info_l1c *info, int* cluster_id, int n_cluster)

{
    int n;
    
    if (n_cluster == 0 || n_cluster > 63)
	return  WL_OUT_OF_RANGE;
				/*  usefull numbers? */
    for (n=0; n<n_cluster; n++)
    {
	if (cluster_id[n] < 0 || cluster_id[n] > 64)
	    return  WL_OUT_OF_RANGE;
    }
				/*  ascending order */
    for (n=0; n<n_cluster-1; n++)
    {
	if (cluster_id[n] >= cluster_id[n+1])
	    return  WL_OUT_OF_RANGE;
    }
				/*  copy to info */
    for (n=0; n<n_cluster; n++)
	info->wanted_clusters[n] = cluster_id[n];

    info->n_wanted_clusters = n_cluster;
    				/*  set pixel limit to all pixels */
				/*  cat be set smaller with
				    set_pix_window or set_wl_window */
    info->cur_pix_end = 8191;
    info->cur_pix_start = 0;
    return OK;
}



	
/*********************************************************************\
 * Close L1c 
\*********************************************************************/

L1C_error closeL1c (info_l1c *info)
{
    free (info->ads_states);
    free (info->mjd_nadir_mds);
    free (info->mjd_limb_mds);
    free (info->idx_nadir_mds);
    free (info->idx_limb_mds);
    if (fclose(info->FILE_l1c) != 0) {
	return FILE_NOT_CLOSED;
  }  
  return OK;
}


/*********************************************************************\
 * Free memory in user_data, allocated in read_next....
\*********************************************************************/

L1C_error free_user_data (user_data *ud)

{
    free (ud->pixel_nr);
    free (ud->pixel_cls);
    free (ud->pixel_ch);
    free (ud->pixel_coadd);
    free (ud->wl);
    free (ud->wl_err);
    free (ud->signal);
    free (ud->signal_err);
    if (ud->n_pmd > 0)
	free (ud->pmd);
    return OK;
}    

L1C_error free_user_data_solar (user_data_solar *ud)
{
    free (ud->pixel_nr);
    free (ud->pixel_ch);
    free (ud->wl);
    free (ud->signal);
    free (ud->signal_err);
    return OK;
}

/*********************************************************************\
 * Read cal_options_GADS
 * kb, 23.04.02
\*********************************************************************/

L1C_error Read_cal_options_GADS (info_l1c *info)
{
    int cluster;

				/* go to GADS cal_options */
    if (fseek(info->FILE_l1c, info->cal_options.offset, SEEK_SET) != -1)
    {
				/* read into memory (include swapping) */
	cal_options_GADS_getbin (info->FILE_l1c, &info->cal_options_GADS);
				/* Set list of included clusters */
	for (cluster=0, info->max_cluster_ids=0; cluster<MAX_CLUSTER; cluster++)
	{
	    if (info->cal_options_GADS.nadir_cluster_flag[cluster] == 0)	  
		continue;		/* Cluster not available */
	    info->cluster_ids [info->max_cluster_ids++] = cluster;
	}
	return OK;
    }
    return NO_GADS_CAL_OPTIONS;
}


	
/*********************************************************************\
 * Compare two dates MJD
 * MJD compare : 0 equal  -1 earlier  1 later
 * kb 23.04.01
\*********************************************************************/

int MJDcompare (MJD *mjd1, MJD *mjd2)
{
    if (mjd1->days < mjd2->days)
	return -1;
    if (mjd1->days > mjd2->days)
	return 1;
				/* Days are equal !! */
    if (mjd1->secnd < mjd2->secnd)
	return -1;
    if (mjd1->secnd > mjd2->secnd)
	return 1;
				/* secnd are equal !! */
    if (mjd1->musec < mjd2->musec)
	return -1;
    if (mjd1->musec > mjd2->musec)
	return 1;
				/* musec are equal !! */
    return 0;
}

/**********************************************************************
 * add 1/16 seconds to MJD
 **********************************************************************/

MJD MJDadd (MJD mjd, int sec_16)
{
    unsigned sec_16_factor = 62500;
    unsigned secnd;
    unsigned musec = mjd.musec;
    mjd.musec = (musec + sec_16 * sec_16_factor) % 1000000;
    secnd = mjd.secnd +
	(musec + sec_16 * sec_16_factor) / 1000000;
    mjd.secnd = secnd % 86400;
    mjd.days = mjd.days + secnd / 86400;
    return mjd;
}

/*********************************************************************\
 * Read complete state info
\*********************************************************************/

L1C_error ADS_state (info_l1c *info)
{
    unsigned num_states, istate;    
    ads_states *state;
    num_states = info->states.num_dsr; /* No. of states */
    if (num_states == 0)
	return NO_ADS_STATES;	
    if (fseek(info->FILE_l1c, info->states.offset, SEEK_SET) != -1)
    {
				/* reserve memory for states */
	info->ads_states = malloc (num_states * sizeof(ads_states));
				/* easy access */
	state = info->ads_states;
				/* read and swap states */
	for (istate=0; istate<num_states;istate++)
	{
	    ads_states_getbin (info->FILE_l1c, &state[istate]);
				/* check the attach flag */	    
	    if ( state[istate].mds_attached != 0 )
	    {			/* ignore states without MDS,  */
		istate--;	/* set counters one back */
		num_states--;
		info->states.num_dsr--;
	    }
	    
#if defined(__DEBUG_L1C__)
	    printf("State %2i %2i %5i %4i %2i\n",
		   istate,
		   state[istate].state_id,
		   state[istate].duration,
		   state[istate].num_clusters,
		   state[istate].num_dsr);
#endif	    
	}
	return OK;
    }
    return NO_ADS_STATES;
}





/*********************************************************************\
 * read solar
\*********************************************************************/

L1C_error read_solar (info_l1c *info,
		      char *sun_spec_id,
		      user_data_solar *ud)
{
  gads_sun_ref sun_ref;
  int pix_start, pix_end, pix_delta;
  int iset;
  int i,n;

				/* are there sun reference data sets? */
  if (info->sun_ref.num_dsr <= 0) {
    return NO_SUN_REF;
  }
				/* position file */
  if (fseek(info->FILE_l1c, info->sun_ref.offset, SEEK_SET) != 0) {
    return NO_SUN_REF;
  }
				/* choose needed pixels */
  pix_start = info->cur_pix_start;
  pix_end = info->cur_pix_end;
  pix_delta = pix_end - pix_start;

  ud->n_wl = pix_delta;
  
				/* allocate memory for output */
  ud->pixel_nr = (unsigned short*) malloc(pix_delta*sizeof(unsigned short));
  ud->pixel_ch = (unsigned short*) malloc(pix_delta*sizeof(unsigned short));
  ud->wl = (float*)malloc(pix_delta*sizeof(float));
  ud->signal = (float*)malloc(pix_delta*sizeof(float));
  ud->signal_err = (float*)malloc(pix_delta*sizeof(float));

  
  
  for (iset=0; iset<info->sun_ref.num_dsr;iset++) {
      
      gads_sun_ref_getbin (info->FILE_l1c, &sun_ref);
/*     sun_ref = ReadSunReference(info->FILE_l1c); */
    
      if (strncmp(sun_ref.id, sun_spec_id, 2) == 0)
      {	
	  for (i=pix_start,n=0; i<pix_end; i++,n++)
	  {
	      ud->pixel_nr[n] = i%1024;
	      ud->pixel_ch[n] = i/1024+1;	      
	      ud->wl[n] = sun_ref.wavel[i];
	      ud->signal[n] = sun_ref.spectrum[i];
	      ud->signal_err[n] = sun_ref.accuracy[i];
	/* also available: sun_ref.etalon[i], sun_ref.precision[i] */
	  }
	  break;			/* no need to read further */
      }
  }
  if (iset == info->sun_ref.num_dsr) {
      printf("no solar data available\n");
      return NO_SUN_REF_DATA;
  }
  /* Copy Sun_spec_id */
  strncpy (ud->sun_spect_id,  sun_ref.id, 3);
  /* Copy orbit info */
  ud->product            = info->user_file_info.product ;           
  ud->software_ver       = info->user_file_info.software_ver ;      
  ud->abs_orbit          = info->user_file_info.abs_orbit;          
  ud->key_data_version   = info->user_file_info.key_data_version ;  
  ud->m_factor_version   = info->user_file_info.m_factor_version ;  
  ud->l1b_product_name   = info->user_file_info.l1b_product_name;   	
  ud->cal_applied        = info->user_file_info.cal_applied;    	
  
#if defined(__DEBUG_L1C__)
  printf("solar spectrum successfully read\n");
#endif  
  return OK;
}



/*********************************************************************\
 * Read MDS (1c) complete
\*********************************************************************/



/*--------------------------------------------------------------------*\
** read MDS (1c cluster data), returns length of DSR
\*--------------------------------------------------------------------*/
unsigned long ReadMDS(FILE* unit, long int offset, L1C_MDS_type type,
		      int Cluster, state_cluster_data* data)

{
  mds_1c_constant mds;

				/* variables for variable part of MDS; 
				   exact size needs to be dynamically
				   allocated */
  int this_size;
#if defined(__DEBUG_L1C__)
  int i;
#endif
  
				/*********************/
				/*  Start of routine */
				/*********************/
#if defined(__DEBUG_L1C__)
  printf ("Cluster: %i %li\n", Cluster, offset);
#endif
				/* go to start of MDS */
  fseek (unit, offset,  SEEK_SET);

				/* read constant part */
  mds_1c_constant_getbin (unit, &mds);

				/* create dynamic variables and read
				   variable part */

				/* pixel id */
  this_size = sizeof(unsigned short)*mds.npixels;
  data->pixel = (unsigned short *)malloc(this_size);
  ushort_array_getbin (unit, data->pixel, mds.npixels);

				/* wavelength */
  this_size = sizeof(float)*mds.npixels;
  data->wl = (float *)malloc(this_size);
  float_array_getbin (unit, data->wl, mds.npixels);

				/* wavelength error */
  data->wl_err = (float *)malloc(this_size);
  float_array_getbin (unit, data->wl_err, mds.npixels);

				/* signal */
  this_size = sizeof(float)*mds.npixels*mds.nobs;
  data->signal = (float *)malloc(this_size);
  float_array_getbin (unit, data->signal, mds.npixels*mds.nobs);

				/* signal error */
  data->signal_err = (float *)malloc(this_size);
  float_array_getbin (unit, data->signal_err, mds.npixels*mds.nobs);

#if defined(__DEBUG_L1C__)
  for (i=0; i<3; i++)
      printf ("%4hi %8.2f %8.2f  %12.2e %12.2e\n",
	      data->pixel[i],
	      data->wl[i],
	      data->wl_err[i],
	      data->signal[i],
	      data->signal_err[i]);
#endif
	     
				/* geolocation (depends on measurement type) */

  switch (type) {

      case NADIR:
	  this_size = mds.nobs*sizeof(GeoN);
	  data->geo_nadir = (GeoN *)malloc(this_size);
	  GeoN_array_getbin (unit, data->geo_nadir, mds.nobs);
	  break;

      case LIMB:
      case OCCULTATION:
	  this_size = mds.nobs*sizeof(GeoL);
	  data->geo_limb = (GeoL *)malloc(this_size);
	  GeoL_array_getbin (unit, data->geo_limb, mds.nobs);
	  break;
    
      case MONITORING:
	  this_size = mds.nobs*sizeof(GeoCal);
	  data->geo_cal = (GeoCal *)malloc(this_size);
	  GeoCal_array_getbin (unit, data->geo_cal, mds.nobs);
      default:	  
	  ;
  }

				/* set output variables */

  data->n_geo = mds.nobs;
  data->n_wl = mds.npixels;
  data->quality = mds.quality;
  data->mds_const = mds;
  data->type = type;
#if defined(__DEBUG_L1C__)
  printf ("%i %i %i %i\n", mds.nobs,mds.npixels, mds.quality ,type );
  printf ("%i\n", mds.length);
#endif   
  return mds.length; 
}

/* used to determine next offset */
  /* remember to free dynamic variables later */
  /*    free(pixel_id); */
  /*    free(wavelength); */
  /*    free(wavelength_error); */
  /*    free(signal); */
  /*    free(signal_error); */

  /*    switch (type) { */
  /*    case NADIR: */
  /*      free(geon); */
  /*      break; */
  /*    case LIMB: */
  /*    case OCCULTATION: */
  /*      free(geol); */
  /*      break; */
  /*    case MONITORING: */
  /*      free(geoc); */
  /*      break; */





/*********************************************************************\
 * Basic routine for reading science data of one state
 * kb 31.07.02
\*********************************************************************/


L1C_error read_next_state (info_l1c *info, L1C_MDS_type type)

{
    int cluster,id,i,n_state;
    
    unsigned long length;

				/* are there data sets? */
  if (info->n_states[type] == 0) {
      printf("No data for type %i!\n", type);
      return NO_DATA;
  }
		     /* check the availability of clusters in state */
				/*  State info list all clusters originally
				    available in 1B product*/
                                /* Calibration options says, what shoul be
				   included */
  info->cur_max_cluster_ids = 0;
				/*  loop of GADS cluster index(0-63) */
  n_state = info->idx_states[type][info->cur_state_nr[type]];

#if defined(__DEBUG_L1C__)
  printf ("n_state= %2i\n", n_state);
#endif
  
  for (cluster=0; cluster<info->max_cluster_ids; cluster++)
  {      
      id = info->cluster_ids [cluster];
				/* loop of state cluster ids (1-64 -> -1 !!) */
      for (i=0; i<64; i++)
      {
	  if (id == info->ads_states[n_state].Clcon[i].id -1 )
	  {
				/*  store match (index: 0-63)*/
	      info->cur_cluster_ids[info->cur_max_cluster_ids++] = id;
	  }
      }
  }
		     /* Something is very wrong, if this happens... */
  if ( info->cur_max_cluster_ids == 0 )
      return NO_CLUSTER_MATCH_IN_STATE;

				/* printf("Offset: %ld\n", offset);*/
  for (cluster=0; cluster<info->cur_max_cluster_ids; cluster++)
  {
#if defined(__DEBUG_L1C__)
      printf ("Read Cluster: %2i %2i   ", cluster, info->cur_cluster_ids[cluster]);
      printf ("offset: %i\n",info->nadir.offset);
#endif      
				/* read next nadir data */
      length = ReadMDS(info->FILE_l1c, info->mds_offset[type], type,
		       info->cur_cluster_ids[cluster],
		       info->st_cl_data + info->cur_cluster_ids[cluster]);

				/* printf("length: %ld, offset: %ld\n", length, offset);*/

				/*  consistency check !!!! */
      if (MJDcompare (&info->st_cl_data[info->cur_cluster_ids[cluster]].mds_const.StartTime,
		      &info->ads_states[n_state].StartTime) != 0)
      {
	  printf ("State <-> MDS mismatch: Fatal programm error!\n");
	  exit (1);
      }
				/* increase offset */
      info->mds_offset[type] += length;
  }

  return OK;
}



/*********************************************************************\
 * Clean up memory from previous read nadir state
 * kb 21.07.02
\*********************************************************************/


L1C_error clear_state (info_l1c *info, L1C_MDS_type type)
{
    int cluster,id;
    if (info->cur_state_nr[type] > 0)
    {
	for (cluster=0; cluster<info->cur_max_cluster_ids; cluster++)
	{
	    id = info->cur_cluster_ids[cluster];
	    free (info->st_cl_data[id].pixel);
	    free (info->st_cl_data[id].wl);
	    free (info->st_cl_data[id].wl_err);
	    free (info->st_cl_data[id].signal);
	    free (info->st_cl_data[id].signal_err);
	    switch (info->st_cl_data[id].type)
	    {
		case NADIR:
		    free (info->st_cl_data[id].geo_nadir);
		    break;
		case LIMB:
		case OCCULTATION:
		    free (info->st_cl_data[id].geo_limb);
		    break;
		case MONITORING:
		    free (info->st_cl_data[id].geo_cal);
		    break;
		default:
		    fprintf(stderr, "L1C_error clear_nadir_state("
			    "info_l1c *info) : Invalid measurement type!\n");
		    exit (1);
		    break;			
	    }
	}
    }
    return OK;
}


/*********************************************************************\
 * Basic routine for reading Nadir next wavelength window
 * kb 23.04.01
\*********************************************************************/
L1C_error calc_cluster_window (info_l1c *info, L1C_MDS_type type)
{
    int n_state,id;
    int cl_start;
    int cl_end;
    int tmp,n;
    int n_cl_start, n_start;
    unsigned short n_cl;
    ads_states state;

#if defined(__DEBUG_L1C__)
    printf ("Entering %s\n", "L1C_error calc_cluster_window (info_l1c *info)");
#endif

    n_state = info->idx_states[type][info->cur_state_nr[type]];
    
    state = info->ads_states[n_state];
				/* Examine cluster definitions */
				/* Which cluster to use? */
    info->max_cur_used_clusters = 0;
				/*  select according to cluster list */
    if (info->n_wanted_clusters > 0)
    {
	for (n_cl = 0; n_cl < info->cur_max_cluster_ids; n_cl++)
	{
	    id = info->cur_cluster_ids[n_cl];
	    for (n = 0; n < info->n_wanted_clusters ; n++)
	    {
				/*  cluster ID in product is 1-64 */
				/*  id in programm starts with 0 as index
				    in C */
		if (id == info->wanted_clusters[n] -1 )
		info->cur_used_clusters[info->max_cur_used_clusters++] = id;
	    }
	}
    }
    else
    {
    for (n_cl = 0; n_cl < info->cur_max_cluster_ids; n_cl++)
    {
	id = info->cur_cluster_ids[n_cl];
	cl_start = (state.Clcon[id].channel-1) * 1024 + state.Clcon[id].pixel_nr;
	cl_end = cl_start + state.Clcon[id].length;

	if (info->cur_pix_start  < cl_end && info->cur_pix_end > cl_start)
	    info->cur_used_clusters[info->max_cur_used_clusters++] = id;
#if defined(__DEBUG_L1C__)
	printf ("Cluster :%2i   %4i-%4i  used %4i\n",
		id, cl_start, cl_end,info->max_cur_used_clusters);
#endif	
    }
    }
				/*  if available cluster not in wl-window*/
				/*  return error value*/
    if (info->max_cur_used_clusters == 0)
	return CLUSTER_WAVELENGTH_MISMATCH;
				/*  sort clusters (Channel 2 is ordered */
				/*  in other direction !!!*/
#if defined(__DEBUG_L1C__)
    for (n_cl = 0; n_cl<info->max_cur_used_clusters; n_cl++)
	printf ("++ %i ", info->cur_used_clusters [n_cl]);
    printf ("\n");
#endif	
    for (n_cl = 0; n_cl<info->max_cur_used_clusters; n_cl++)
    {
	for (n = n_cl+1; n <info->max_cur_used_clusters; n++)
	{
	    id = info->cur_used_clusters[n_cl];
	    n_cl_start = (state.Clcon[id].channel-1) * 1024 +
		state.Clcon[id].pixel_nr;
	    id = info->cur_used_clusters[n];
	    n_start = (state.Clcon[id].channel-1) * 1024 +
		state.Clcon[id].pixel_nr;
#if defined(__DEBUG_L1C__)	    
	    printf ("%i %i ++", n_start, n_cl_start);
#endif
	    if  ( n_start < n_cl_start )
	    {
		tmp = info->cur_used_clusters[n];
		info->cur_used_clusters [n] = info->cur_used_clusters [n_cl];
		info->cur_used_clusters [n_cl] = tmp;
	    }
	}
    }
#if defined(__DEBUG_L1C__)
    for (n_cl = 0; n_cl<info->max_cur_used_clusters; n_cl++)
	printf ("++ %i ", info->cur_used_clusters [n_cl]);
    printf ("\n");
#endif	
    
				/* Determine co-adding factors and
				   Geol. to use*/
				/*    longest int_time in pix_window */
    info->geo_cur_used_clusters = info->cur_used_clusters[0];
    for (n_cl = 1; n_cl<info->max_cur_used_clusters; n_cl++)
    {
	id = info->cur_used_clusters[n_cl];
	if (state.Clcon[id].int_time > state.Clcon[info->geo_cur_used_clusters].int_time)
	    info->geo_cur_used_clusters = id;
    }    

    info->sum_pix_cur_used_clusters = 0;
    for (n_cl = 0; n_cl<info->max_cur_used_clusters; n_cl++)
    {
	id = info->cur_used_clusters[n_cl];
				/*   calc co-adding factors  */
	info->coadd_cur_used_clusters[n_cl] =
	    state.Clcon[info->geo_cur_used_clusters].int_time / state.Clcon[id].int_time;
				/*   restrict pixel window  */
	cl_start = (state.Clcon[id].channel-1) * 1024 + state.Clcon[id].pixel_nr;
	cl_end = cl_start + state.Clcon[id].length;

	info->pix_start_cur_used_clusters[n_cl] = 0;
	if ( cl_start < info->cur_pix_start)
	    info->pix_start_cur_used_clusters[n_cl] =
		info->cur_pix_start - cl_start;

	info->pix_end_cur_used_clusters[n_cl] = state.Clcon[id].length;

	if ( cl_end > info->cur_pix_end)
	    info->pix_end_cur_used_clusters[n_cl] -=
		cl_end - info->cur_pix_end;

	info->sum_pix_cur_used_clusters +=
	    info->pix_end_cur_used_clusters[n_cl] -
	    info->pix_start_cur_used_clusters[n_cl];
#if defined(__DEBUG_L1C__)
	printf ("Custer :%2i   %4i-%4i  used pix  %4i %4i sum %4i\n",
		id, cl_start, cl_end,
		info->pix_start_cur_used_clusters[n_cl],
		info->pix_end_cur_used_clusters[n_cl],
		info->sum_pix_cur_used_clusters
		);
#endif
    }

				/* Calculate number of readouts for pix-window */
	info->cur_max_readout_in_state =
	state.duration / state.Clcon[info->geo_cur_used_clusters].int_time;
	info->cur_it = state.Clcon[info->geo_cur_used_clusters].int_time;
#if defined(__DEBUG_L1C__)
	printf ("cur_max_read_out_in_state %i",
		info->cur_max_readout_in_state);
#endif
	info->cur_max_readout_in_state =
	info->st_cl_data[info->geo_cur_used_clusters].mds_const.nobs;
#if defined(__DEBUG_L1C__)
	printf ("cur_max_read_out_in_state %i",
		info->cur_max_readout_in_state);
#endif
				/*  calculate number of PMD measurements */
				/*  PMD 1/32s    IT in 1/16s */
	info->cur_num_pmd = info->cur_it * 2;
					/* Test output */
#if defined(__DEBUG_L1C__)
	printf ("cur_max_readout_in_state %i %i %i \n",
		info->cur_max_readout_in_state,
	    state.duration ,
	    state.Clcon[info->geo_cur_used_clusters].int_time);
	printf ("cur_num_pmd, info->cur_it %i %i ",
		info->cur_num_pmd, info->cur_it);
#endif
    return OK;
}

/*********************************************************************\
 * Coadding data and errors
 * kb 23.04.01
\*********************************************************************/

L1C_error coadd_signal (unsigned int n_wl, unsigned int n_coadd,
			float *signal,           /* Input: rad[n_coadd*n_wl] */
			float *signal_err,       /* Input: rad_err[n_coadd*n_wl] */
			float *add_signal,       /* Output: rad[n_wl] */
			float *add_signal_err)   /* Output: rad_err[n_wl] */
{
    int nw, nc;
    for (nw = 0; nw < n_wl; nw++)
    {
	add_signal[nw] = 0.0;
	add_signal_err[nw] = 0.0;
	for (nc = 0; nc < n_coadd; nc++)
	{
	    add_signal[nw] += signal[ nc*n_wl + nw ];
	    add_signal_err[nw] += signal_err[ nc*n_wl + nw ] * signal_err[ nc*n_wl + nw ];
	}
	add_signal[nw] = add_signal[nw] / (float)n_coadd;
				/* !!!!!!!!!!! Check coadding errors : OK*/
	add_signal_err[nw] = sqrt (add_signal_err[nw]) / (float)n_coadd;
    }
    return OK;
}

/*********************************************************************\
 * Basic routine for reading Nadir next wavelength window
 * kb 23.04.01
\*********************************************************************/

L1C_error read_next_mds (info_l1c *info,
			   user_data *ud, L1C_MDS_type type)
{
    unsigned int n_pix, n_pix_tmp, n_cl, i_coadd, n_pmd, i;
    unsigned int pix_start, pix_end;
    unsigned int n_tmp_pix, n_coadd;
    unsigned int n_ro, cur_pix, id;
    int n_state;
    float *tmp_signal;
    float *tmp_signal_err;
    L1C_error errflag;
    state_cluster_data *data;
#if defined(__DEBUG_L1C__)
	printf ("read_next_nadir");
	printf ("cur_state_nr[]: %i mds_offset: %i  read_out %i\n",
		info->cur_state_nr[type], info->mds_offset[NADIR],
		info->cur_readout_in_state );
#endif		      
				/* if next state, read state data/info */
    if (info->cur_readout_in_state == 0)
    {
				/* Free previous allocated memory */
	clear_state(info, type);
	if (info->cur_state_nr[type] == info->n_states[type])
	    return END_OF_NADIR_STATES;
				/* read state in info->st_cl_data */
	read_next_state (info,type);
				/* define used clusters for this state */
	if ( (errflag = calc_cluster_window (info,type)) != OK)
	    return errflag;	/* wl-window - cluster mismatch */
				/* read pmds  */
	read_next_pmd_state (info, type);
    }
    n_ro = info->cur_readout_in_state;
    n_state = info->idx_states[type][info->cur_state_nr[type]];
    
    ud->n_wl = info->sum_pix_cur_used_clusters;
				/* allocate memory */
    ud->pixel_nr = malloc (sizeof (unsigned short) * ud->n_wl);
    ud->pixel_cls = malloc (sizeof (unsigned short) * ud->n_wl);
    ud->pixel_ch = malloc (sizeof (unsigned short) * ud->n_wl);
    ud->pixel_coadd = malloc (sizeof (unsigned short) * ud->n_wl);
    ud->wl = malloc (sizeof (float) * ud->n_wl);
    ud->wl_err = malloc (sizeof (float) * ud->n_wl);
    ud->signal = malloc (sizeof (float) * ud->n_wl);
    ud->signal_err = malloc (sizeof (float) * ud->n_wl);
    if (info->n_pmd_mds[type] > 0)
    {
	ud->n_pmd = info->cur_num_pmd;
	ud->pmd = malloc (sizeof (user_data_pmd) * ud->n_pmd);
    }
    else
    {
	ud->n_pmd = 0;
	ud->pmd = NULL;
    }
				/* collect data */
    cur_pix=0;
				/*  Loop over used clusters */
    for (n_cl=0; n_cl < info->max_cur_used_clusters; n_cl++)
    {
	id = info->cur_used_clusters[n_cl];
	data = info->st_cl_data + id;
	pix_start = info->pix_start_cur_used_clusters[n_cl];
	pix_end = info->pix_end_cur_used_clusters[n_cl];
	n_tmp_pix = pix_end - pix_start;
	n_coadd = info->coadd_cur_used_clusters[n_cl];
				/* co_add signal / err */
	tmp_signal = malloc (sizeof(float) * n_tmp_pix * n_coadd);
	tmp_signal_err = malloc (sizeof(float) * n_tmp_pix * n_coadd);
	for (i_coadd = 0; i_coadd < n_coadd; i_coadd++)
	{
	    for (n_pix = pix_start, n_pix_tmp=0;
		 n_pix < pix_end;
		 n_pix++, n_pix_tmp++)
	    {
/* #if defined(__DEBUG_L1C__) */
/* 		printf ("n_pix, data->n_wl, i_coadd , n_ro , n_coadd" */
/* 			"   %3i %3i %3i %3i %3i  -> %5i ( %4i )\n", */
/* 			n_pix, data->n_wl, i_coadd , n_ro, n_coadd, */
/* 			n_pix + data->n_wl*(i_coadd + n_ro*n_coadd), */
/* 			n_pix_tmp + n_tmp_pix*i_coadd); */
/* #endif		 */
		tmp_signal[n_pix_tmp + n_tmp_pix*i_coadd] =
		    data->signal[n_pix +
				data->n_wl* (i_coadd + n_ro*n_coadd)];
				/*cur     cur coadd           readouts */
		tmp_signal_err[n_pix_tmp + n_tmp_pix*i_coadd] =
		    data->signal_err[n_pix +
				    data->n_wl * (i_coadd + n_ro*n_coadd)];
	    }
	}
	coadd_signal (n_tmp_pix, n_coadd, tmp_signal, tmp_signal_err,
		      ud->signal+cur_pix, ud->signal_err+cur_pix);
				/* collect other parts, identical for all readouts */
	for (n_pix = pix_start, n_pix_tmp=0;
	     n_pix < pix_end;
	     n_pix++, n_pix_tmp++)
	{
	    ud->pixel_nr [ cur_pix+n_pix_tmp ] = data->pixel[n_pix];
	    ud->pixel_cls [ cur_pix+n_pix_tmp ] = id + 1;
	    ud->pixel_ch [ cur_pix+n_pix_tmp ] = 
		info->ads_states[n_state].Clcon[id].channel;

	    ud->pixel_coadd [ cur_pix+n_pix_tmp ] = n_coadd;
	    ud->wl [ cur_pix+n_pix_tmp ] = data->wl[n_pix];
	    ud->wl_err [ cur_pix+n_pix_tmp ] = data->wl_err[n_pix];
	}
	cur_pix += n_tmp_pix;
    }
    switch (type) {

      case NADIR:
	  ud->geo_nadir = info->st_cl_data[info->geo_cur_used_clusters].geo_nadir[n_ro];
	  break;
      case LIMB:
      case OCCULTATION:
	  ud->geo_limb = info->st_cl_data[info->geo_cur_used_clusters].geo_limb[n_ro];
	  break;
    
      case MONITORING:
	  ud->geo_cal = info->st_cl_data[info->geo_cur_used_clusters].geo_cal[n_ro];
	  break;
      default:	  
	  ;
  }

/***********************************************************************
Description of Limb/Occultation Time calculations (IODD 0-1, p 57)
------------------------------------------------------------------------
    The measurement duration of a Limb state is 59.0625 seconds. This
    may be calculated as follows: one azimuth scan has a net
    integration time of 1.5 seconds and the adjustment of the
    elevation mirror for the next azimuth scan takes 3 BCPSs
    corresponding to 0.1875 seconds giving a total of 1.6875 seconds
    for one azimuth scan. 34 azimuth scan plus one dark measurement
    into deep space (which takes the same time as one azimuth scan)
    gives a state duration of: 35  1.6875 s = 59.0625 s.
**********************************************************************/
    
    /* Calculate date */
    switch (type)
    {
	case NADIR:	    
	    ud->mjd = MJDadd(info->ads_states[n_state].StartTime ,
			     info->cur_it * n_ro);	    
	    break;
	case LIMB:
	case OCCULTATION:
	    ud->mjd = MJDadd(info->ads_states[n_state].StartTime ,
			     info->cur_it * n_ro +
			     ((info->cur_it * n_ro) / SCAN_LIMB_DURATION)
			     * SCAN_LIMB_STEP_DURATION);
	    printf ("---> %3d %3d %3d %3d %3d  %3d\n" , info->cur_it, n_ro,
		    info->cur_it * n_ro,
		    ((info->cur_it * n_ro) / SCAN_LIMB_DURATION),
		    ((info->cur_it * n_ro) / SCAN_LIMB_DURATION)
		    * SCAN_LIMB_STEP_DURATION,
		    info->cur_it * n_ro +
			     ((info->cur_it * n_ro) / SCAN_LIMB_DURATION)
		    * SCAN_LIMB_STEP_DURATION);
	    break;
	default:
	    ud->mjd =  info->ads_states[n_state].StartTime;
    }
    UTC_String (&ud->mjd, ud->date);
    ud->int_time = info->cur_it / 16.0;

    /* Copy state info  */
    ud->state_id = info->ads_states[n_state].state_id;
    UTC_String (&info->ads_states[n_state].StartTime,
		ud->state_date) ;

    /* Copy orbit info */
    ud->product            = info->user_file_info.product ;           
    ud->software_ver       = info->user_file_info.software_ver ;      
    ud->abs_orbit          = info->user_file_info.abs_orbit;          
    ud->key_data_version   = info->user_file_info.key_data_version ;  
    ud->m_factor_version   = info->user_file_info.m_factor_version ;  
    ud->l1b_product_name   = info->user_file_info.l1b_product_name;   	
    ud->cal_applied        = info->user_file_info.cal_applied;    	
    
    /*  Copy PMD data */
    if (info->n_pmd_mds[type] > 0)
	for (n_pmd=0; n_pmd < ud->n_pmd; n_pmd++)
	{
	    for (i=0; i< 7; i++)
	    {
		ud->pmd[n_pmd].pmd[i] =
		    info->nadir_pmd_data.int_pmd [n_ro * ud->n_pmd * 7
						  + n_pmd*7 + i];
	    }
	}
    /* Copy counter  */
    ud->n_readout = info->cur_readout_in_state;
    ud->n_state = info->cur_state_nr[type];
    /*  update counter for readout in state */
    info->cur_readout_in_state = (info->cur_readout_in_state+1) %
	info->cur_max_readout_in_state;
    /* update state counter, if necessary */
    if (  info->cur_readout_in_state == 0 )
	info->cur_state_nr[type]++;
    return OK;
}


    


L1C_error read_next_pmd_state (info_l1c *info, L1C_MDS_type type)

{
    unsigned long length;
    if (info->n_pmd_mds[type] <= 0) {
#if defined(__DEBUG_L1C__)
	printf("No PMD data for type %i!\n", type);
#endif
	return NO_PMD_DATA;
    }
#if defined(__DEBUG_L1C__)
    printf ("Read PMD\n");
#endif
    length =  Read_PMD_MDS (info->FILE_l1c, info->mds_pmd_offset[type], &info->nadir_pmd_data, type);

    info->mds_pmd_offset[type] += length;

    return OK;
}



unsigned long Read_PMD_MDS (FILE* unit,
			    long int offset,
			    nadir_pmd *pmd,
			    L1C_MDS_type type)
{
    				/* go to start of MDS */
    fseek (unit, offset,  SEEK_SET);
				/*  */
    MJD_getbin (unit, &pmd->dsr_time);
    uint_getbin (unit, &pmd->dsr_length);
    uchar_getbin (unit, &pmd->quality_flag);
    float_getbin (unit, &pmd->orb_phase);
    ushort_getbin (unit, &pmd->meas_cat);
    ushort_getbin (unit, &pmd->state_id);
    ushort_getbin (unit, &pmd->dur_scan_phase );
    ushort_getbin (unit, &pmd->num_pmd);
    ushort_getbin (unit, &pmd->num_geo);
    pmd->int_pmd = (float*) malloc (pmd->num_pmd * sizeof(float));
    
    float_array_getbin (unit, pmd->int_pmd, pmd->num_pmd);
    /* to be completed .... */

    switch (type) {

	case NADIR:
	    pmd->geo_nadir = (GeoN*) malloc (pmd->num_pmd * sizeof(GeoN));
	    GeoN_array_getbin (unit, pmd->geo_nadir, pmd->num_geo);
	    break;
	case LIMB:
	case OCCULTATION:
	    pmd->geo_limb = (GeoL*) malloc (pmd->num_pmd * sizeof(GeoL));
	    GeoL_array_getbin (unit, pmd->geo_limb, pmd->num_geo);
	    break;
	case MONITORING:
	default:	  
	  ;
    }
    return pmd->dsr_length;
}



L1C_error Read_MPH (FILE* unit, MPH *mph)

{
    int n;
    char *tmp = (char*) mph;
    for (n=0; n<sizeof(MPH); n++)
	tmp[n] = 0;
    
    fscanf(unit,
	   "PRODUCT=\"%62c\"\n"
	   "PROC_STAGE=%1c\n"
	   "REF_DOC=\"%23c\"\n"
/*	   "%40c\n" */
	   "ACQUISITION_STATION=\"%20c\"\n"
	   "PROC_CENTER=\"%6c\"\n"
	   "PROC_TIME=\"%27c\"\n"
	   "SOFTWARE_VER=\"%14c\"\n"
/*	   "%40c\n" */
	   "SENSING_START=\"%27c\"\n"
	   "SENSING_STOP=\"%27c\"\n"
/*	   "%40c\n" */
	   "PHASE=%1c\n"
	   "CYCLE=%4c\n"
	   "REL_ORBIT=%6c\n"
	   "ABS_ORBIT=%6c\n"
	   "STATE_VECTOR_TIME=\"%27c\"\n"
	   "DELTA_UT1=%8c<s>\n"
	   "X_POSITION=%12c<m>\n"
	   "Y_POSITION=%12c<m>\n"
	   "Z_POSITION=%12c<m>\n"
	   "X_VELOCITY=%12c<m/s>\n"
	   "Y_VELOCITY=%12c<m/s>\n"
	   "Z_VELOCITY=%12c<m/s>\n"
	   "VECTOR_SOURCE=\"%2c\"\n"
/*	   "%40c\n" */
	   "UTC_SBT_TIME=\"%27c\"\n"
	   "SAT_BINARY_TIME=%11c\n"
	   "CLOCK_STEP=%11c<ps>\n"
/*	   "%32c\n" */
	   "LEAP_UTC=\"%27c\"\n"
	   "LEAP_SIGN=%4c\n"
	   "LEAP_ERR=%1c\n"
/*	   "%40c\n" */
	   "PRODUCT_ERR=%1c\n"
	   "TOT_SIZE=%21c<bytes>\n"
	   "SPH_SIZE=%11c<bytes>\n"
	   "NUM_DSD=%11c\n"
	   "DSD_SIZE=%11c<bytes>\n"
	   "NUM_DATA_SETS=%11c\n",
	   mph->product,
	   mph->proc_stage,
	   mph->ref_doc ,
/*	   mph->spare_1 , */
	   mph->acquisition_station ,
	   mph->proc_center ,
	   mph->proc_time ,
	   mph->software_ver ,
/*	   mph->spare_2 ,*/
	   mph->sensing_start ,
	   mph->sensing_stop ,
/*	   mph->spare_3 , */
	   mph->phase ,
	   mph->cycle ,
	   mph->rel_orbit ,
	   mph->abs_orbit ,
	   mph->state_vector_time ,
	   mph->delta_ut1 ,
	   mph->x_position ,
	   mph->y_position ,
	   mph->z_position ,
	   mph->x_velocity ,
	   mph->y_velocity ,
	   mph->z_velocity ,
	   mph->vector_source ,
/*	   mph->spare_4 , */
	   mph->utc_sbt_time ,
	   mph->sat_binary_time ,
	   mph->clock_step ,
/*	   mph->spare_5 , */
	   mph->leap_utc ,
	   mph->leap_sign ,
	   mph->leap_err ,
/*	   mph->spare_6 , */
	   mph->product_err ,
	   mph->tot_size ,
	   mph->sph_size ,
	   mph->num_dsd ,
	   mph->dsd_size ,
	   mph->num_data_sets 
/*	   mph->spare_7 */ );

    return OK;
}


L1C_error Read_SPH (FILE* unit, SPH *sph)

{
    int n;
    char *tmp = (char*) sph;
    for (n=0; n<sizeof(SPH); n++)
	tmp[n] = 0;
    
    fscanf(unit,
	   "SPH_DESCRIPTOR=\"%28c\"\n"
	   "STRIPLINE_CONTINUITY_INDICATOR=%4c\n"
	   "SLICE_POSITION=%4c\n"
	   "NUM_SLICES=%4c\n"
	   "START_TIME=\"%27c\"\n"
	   "STOP_TIME=\"%27c\"\n"
	   "START_LAT=%11c<10-6degN>\n"
	   "START_LONG=%11c<10-6degE>\n"
	   "STOP_LAT=%11c<10-6degN>\n"
	   "STOP_LONG=%11c<10-6degE>\n"                                                  
	   "KEY_DATA_VERSION=\"%5c\"\n"
	   "M_FACTOR_VERSION=\"%5c\"\n"
	   "SPECTRAL_CAL_CHECK_SUM=\"%4c\"\n"
	   "SATURATED_PIXEL=\"%4c\"\n"
	   "DEAD_PIXEL=\"%4c\"\n"
	   "DARK_CHECK_SUM=\"%4c\"\n"
	   "NO_OF_NADIR_STATES=%4c\n"
	   "NO_OF_LIMB_STATES=%4c\n"
	   "NO_OF_OCCULTATION_STATES=%4c\n"
	   "NO_OF_MONI_STATES=%4c\n"
	   "NO_OF_NOPROC_STATES=%4c\n"
	   "COMP_DARK_STATES=%4c\n"
	   "INCOMP_DARK_STATES=%4c\n",
	   sph->sph_descriptor ,
	   sph->stripline_continuity_indicator ,
	   sph->slice_position ,
	   sph->num_slices ,
	   sph->start_time ,
	   sph->stop_time ,
	   sph->start_lat ,
	   sph->start_long ,
	   sph->stop_lat ,
	   sph->stop_long ,
	   sph->key_data_version ,
	   sph->m_factor_version ,
	   sph->spectral_cal_check_sum ,
	   sph->saturated_pixel ,
	   sph->dead_pixel ,
	   sph->dark_check_sum ,
	   sph->no_of_nadir_states ,
	   sph->no_of_limb_states ,
	   sph->no_of_occultation_states ,
	   sph->no_of_moni_states ,
	   sph->no_of_noproc_states ,
	   sph->comp_dark_states ,
	   sph->incomp_dark_states );
    return OK;
};


L1C_error Read_DSD (FILE* unit, DSD *dsd)
{
    int err;
    char *str_ptr;
    err=fscanf(unit,        
	   "DS_NAME=\"%28c\"\n"
	   "DS_TYPE=%1c\n"
	   "FILENAME=\"%62c\"\n"
	   "DS_OFFSET=+%20u<bytes>\n"
	   "DS_SIZE=%21u<bytes>\n"
	   "NUM_DSR=%11u\n"
	   "DSR_SIZE=%11d<bytes>\n",
	   dsd->name,
	   &dsd->type,
	   dsd->filename,
	   &dsd->offset,
	   &dsd->size,
	   &dsd->num_dsr,
	   &dsd->dsr_size);

    dsd->name[28]='\0';
    if ( (str_ptr = strpbrk(dsd->name, " ") ) != NULL)
	*str_ptr='\0';

    dsd->filename[62]='\0';
    if ( (str_ptr = strpbrk(dsd->filename, " ") ) != NULL)
	*str_ptr='\0';
    
    return OK;
}
                                

